/*
 * afetch 1.1
 * Jrgen Hgg 1994
 * <jh@efd.lth.se>
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "nntp.h"
#include "nntpclnt.h"

void	get_active();
void	select_group(char *);
void	get_xhdr(char *);
void	get_article(char *);
void	select_and_get_article(char *);
void	usage();
int	f_newsrc = 0, f_active = 0, f_id = 0, f_mid = 0, f_subject = 0,
	n_art = 0, first_art = 0, last_art = 0;
char	*pipe_thru = NULL, *file_prefix = NULL, cur_group[512];
char	buf[512];

main(int argc, char *argv[])
{
	char		*server = NEWS_SERVER;
	int		i, c, res, f_active = 0;
	char		tmp[128], *group = NULL, *xhdr = NULL;
	extern char	*optarg;
	extern int	optind;

	while ((c = getopt(argc, argv, "Aaf:g:lmnp:s:x:")) != -1)
		switch(c)
		{
		case 'a':
			f_active++;
			break;
		case 'A':
			f_id++;
			break;
		case 'f':
			file_prefix = optarg;
			break;
		case 'g':
			group = optarg;
			break;
		case 'l':
			f_subject++;
			break;
		case 'm':
			f_mid++;
			break;
		case 'n':
			f_newsrc++;
			break;
		case 'p':
			pipe_thru = optarg;
			break;
		case 'x':
			xhdr = optarg;
			break;
		case 's':
			server = optarg;
			break;
		case '?':
			usage();
			exit(1);
		}

	if (!f_active && !f_id && !group)
	{
		usage();
		exit(1);
	}

	if ((res = server_init(server)) < 0)
	{
		fprintf(stderr, "Couldn't connect to %s news server,"
			" try again later.\n", server);
		exit(1);
	}
	if (res > 500)
	{
		printf("Sorry, no permission to use %s\n", server);
		exit(1);
	}

	if (f_active)
		get_active();
	else if (f_id)
	{
		for (i = optind; i < argc; i++)
			get_article(argv[i]);
	}
	else if (f_mid)
	{
		select_group(group);
		sprintf(tmp, "Message-Id %d-%d", first_art, last_art);
		get_xhdr(tmp);
	}
	else if (f_subject)
	{
		select_group(group);
		sprintf(tmp, "Subject %d-%d", first_art, last_art);
		get_xhdr(tmp);
	}
	else if (xhdr)
	{
		select_group(group);
		get_xhdr(xhdr);
	}
	else
	{
		select_group(group);
		if (optind == argc)
			printf("%s from %d to %d (%d articles)\n",
				cur_group, first_art, last_art, n_art);
		else
			for (i = optind; i < argc; i++)
				select_and_get_article(argv[i]);
	}
	close_server();
}

void	select_and_get_article(char *s)
{
	int	to, i;
	char	*p, tmp[64];

	if (atoi(s) > 0 && (p = strchr(s, '-')))
	{
		*(p++) = '\0';
		to = atoi(p);
		for (i = atoi(s); i <= to; i++)
		{
			sprintf(tmp, "%d", i);
			get_article(tmp);
		}
	}
	else
		get_article(s);
}

void	get_active()
{
	char	*p;

	put_server("list");
	if (get_server(buf, 512) < 0)
	{
		fprintf(stderr, "get_server returned error.\n");
		exit(1);
	}
	if (atoi(buf) != 215)
	{
		fprintf(stderr, "LIST returned %s\n", buf);
		exit(1);
	}
	while(1)
	{
		if (get_server(buf, 512) < 0)
		{
			fprintf(stderr, "get_server returned error.\n");
			exit(1);
		}
		if (!strcmp(buf, "."))
			break;
		if (f_newsrc)
		{
			if (p = strchr(buf, ' '))
				strcpy(p, ":");
		}
		puts(buf);
	}
}
	
void	select_group(char *group)
{
	char	s[256];
	int	x;

	sprintf(buf, "group %s", group);
	put_server(buf);
	if (get_server(buf, 512) < 0)
	{
		fprintf(stderr, "get_server returned error.\n");
		exit(1);
	}
	if (atoi(buf) != 211)
	{
		fprintf(stderr, "GROUP %s returned %s\n", group, buf);
		return;
	}
	sscanf(buf, "%d %d %d %d %s",
		&x, &n_art, &first_art, &last_art, cur_group);
}
	
void	get_xhdr(char *arg)
{
	sprintf(buf, "xhdr %s", arg);
	put_server(buf);
	if (get_server(buf, 512) < 0)
	{
		fprintf(stderr, "get_server returned error.\n");
		exit(1);
	}
	if (atoi(buf) != 221)
	{
		fprintf(stderr, "XHDR %s returned %s\n", arg, buf);
		return;
	}
	while(1)
	{
		if (get_server(buf, 512) < 0)
		{
			fprintf(stderr, "get_server returned error.\n");
			exit(1);
		}
		if (!strcmp(buf, "."))
			break;
		puts(buf);
	}
}

void	get_article(char *s)
{
	int	res, i;
	FILE	*fp = NULL;
	char	ftmp[256];

	sprintf(buf, "article %s", s);
	put_server(buf);
	if (get_server(buf, 512) < 0)
	{
		fprintf(stderr, "get_server returned error.\n");
		exit(1);
	}
	if ((res = atoi(buf)) > 299)
	{
		fprintf(stderr, "ARTICLE %s returned %s\n", s, buf);
		return;
	}
	if (res > 220)
		fprintf(stderr, "ARTICLE %s returned %s\n", s, buf);
	if (pipe_thru)
	{
		if (!(fp = popen(pipe_thru, "w")))
		{
			perror(pipe_thru);
			exit(1);
		}
	}
	else if (file_prefix)
	{
		sprintf(ftmp, "%s%s", file_prefix, s);
		if (!(fp = fopen(ftmp, "w")))
		{
			perror(file_prefix);
			exit(1);
		}
	}
		
	while(1)
	{
		if (get_server(buf, 512) < 0)
		{
			fprintf(stderr, "get_server returned error.\n");
			exit(1);
		}
		if (!strcmp(buf, "."))
			break;
		if (fp)
			fprintf(fp, "%s\n", buf);
		else
			puts(buf);
	}
	if (pipe_thru)
		pclose(fp);
	else if (file_prefix)
		fclose(fp);
}
void usage()
{
	fprintf(stderr, "Usage: afetch  [options] artnumber ...\n"
		"Number of article may have a range, from-to.\n\n"
		"\t-A\tUse 'Message-ID' instead of articlenumber.\n"
		"\t-a\tGet active-list\n"
		"\t-f prefix\tSave to file(s) with prefix.\n"
		"\t-n\tGet a complete .newsrc.\n"
		"\t-s\tUse other news-host.\n"
		"\t-g name\tGet from newsgroup 'name'.\n"
		"\nGroup must be selected for the following options.\n"
		"\t-l\tShow all subjects.\n"
		"\t-m\tShow all message-id's.\n"
		"\t-x arg\tRun 'XHDR arg'.\n");
}
